import datetime

from lpworkitems import models_roadmap
from utils import unicode_or_None


class CollectorStore(object):

    def __init__(self, store, base_url, error_collector):
        self.store = store
        self.base_url = base_url
        self.error_collector = error_collector

    def _clear_all(self, *find_args):
        self.store.find(*find_args).remove()

    def clear_lanes(self):
        self._clear_all(models_roadmap.Lane)

    def clear_cards(self):
        self._clear_all(models_roadmap.Card)

    def store_lane(self, lane):
        self.store.add(lane)

    def store_card(self, card):
        self.store.add(card)

    def clear_todays_blueprint_daily_count_per_state(self):
        self._clear_all(
            models_roadmap.BlueprintDailyCountPerState,
            models_roadmap.BlueprintDailyCountPerState.day == datetime.date.today())

    def store_roadmap_bp_count_per_state(self):
        query = """
            SELECT implementation, lane_id, count(*)
            FROM specs
            JOIN meta on spec = specs.name
            JOIN card on roadmap_id = value
            WHERE key = 'Roadmap id'
            GROUP BY implementation, lane_id
            """
        day = datetime.date.today()
        result = self.store.execute(query)
        for status, lane_id, count in result:
            obj = models_roadmap.BlueprintDailyCountPerState()
            obj.day = day
            obj.status = status
            obj.lane_id = lane_id
            obj.count = count
            self.store.add(obj)

    def lane_is_collected(self, lane_id):
        return self.store.find(models_roadmap.Lane, models_roadmap.
                               Lane.lane_id == lane_id).one() is not None


def get_json_item(data, item_name):
    item = data[item_name]
    if item is not None:
        item = item.strip()
    return unicode_or_None(item)


def lookup_kanban_priority(numeric_priority):
    priority_lookup = {-1: "low",
                        0: "normal",
                        1: "high"}
    assert numeric_priority in priority_lookup, (
        "Priority '%s' is unknown." % numeric_priority)
    return unicode_or_None(priority_lookup[numeric_priority])
