import datetime

from lpworkitems.collect_roadmap import (
    CollectorStore,
    get_json_item,
    )
from lpworkitems.models_roadmap import BlueprintDailyCountPerState
from lpworkitems.error_collector import (
    ErrorCollector,
    )
from lpworkitems.testing import TestCaseWithFakeLaunchpad


class CollectorTests(TestCaseWithFakeLaunchpad):

    def setUp(self):
        super(CollectorTests, self).setUp()
        self.error_collector = ErrorCollector()
        self.collector = CollectorStore(
            self.store, self.lp, self.error_collector)

    def assertClears(self, cls, fn):
        self.assertTrue(self.store.find(cls).count() > 0)
        fn()
        self.assertEqual(0, self.store.find(cls).count())

    def test_clear_todays_blueprint_daily_count_per_state(self):
        self.factory.make_blueprint_daily_count_per_state(
            day=datetime.date.today())
        self.assertClears(
            BlueprintDailyCountPerState,
            self.collector.clear_todays_blueprint_daily_count_per_state)

    def test_store_roadmap_bp_count_per_state(self):
        bp = self.factory.make_blueprint()
        card = self.factory.make_card()
        meta = self.factory.make_meta(
            key=u'Roadmap id', value=card.roadmap_id, blueprint=bp)
        self.collector.store_roadmap_bp_count_per_state()
        self.assertEqual(
            1, self.store.find(BlueprintDailyCountPerState).count())
        entry = self.store.find(BlueprintDailyCountPerState).one()
        self.assertEqual(1, entry.count)
        self.assertEqual(card.lane_id, entry.lane_id)
        self.assertEqual(bp.implementation, entry.status)

    # XXX Add tests for the roadmap classes.


class RoadmapUtilsTests(TestCaseWithFakeLaunchpad):

    def setUp(self):
        super(RoadmapUtilsTests, self).setUp()
        self.json_data = {"data": "Text",
                          "whitespace": " Text  ",
                          "none": None
                          }

    def test_get_json_data_unicode(self):
        item = get_json_item(self.json_data, 'data')
        self.assertEquals(item, u'Text')

    def test_get_json_data_whitespace(self):
        item = get_json_item(self.json_data, 'whitespace')
        self.assertEquals(item, u'Text')

    def test_get_json_data_none(self):
        item = get_json_item(self.json_data, 'none')
        self.assertEquals(item, None)
