from report_tools import (
    card_blueprints,
    card_blueprints_by_status,
)

card_health_checks = []


def register_health_check(cls):
    card_health_checks.append(cls)
    return cls


class HealthCheck(object):
    NOT_APPLICABLE = 'n/a'
    OK = 'OK'
    NOT_OK = 'Not OK'
    name = 'Base check, not to be used'

    @classmethod
    def applicable(cls, card, store=None):
        raise NotImplementedError()

    @classmethod
    def check(cls, card, store=None):
        raise NotImplementedError()

    @classmethod
    def execute(cls, card, store=None):
        if cls.applicable(card, store):
            if cls.check(card, store):
                return cls.OK
            else:
                return cls.NOT_OK
        else:
            return cls.NOT_APPLICABLE


@register_health_check
class DescriptionHealthCheck(HealthCheck):
    name = 'Has description'

    @classmethod
    def applicable(cls, card, store=None):
        return True

    @classmethod
    def check(cls, card, store=None):
        return card.description is not None


@register_health_check
class CriteriaHealthCheck(HealthCheck):
    name = 'Has acceptance criteria'

    @classmethod
    def applicable(cls, card, store=None):
        return True

    @classmethod
    def check(cls, card, store=None):
        return card.acceptance_criteria is not None


@register_health_check
class BlueprintsHealthCheck(HealthCheck):
    name = 'Has blueprints'

    @classmethod
    def applicable(cls, card, store):
        return card.status == 'Ready'

    @classmethod
    def check(cls, card, store):
        return len(card_blueprints(store, card.roadmap_id)) > 0


@register_health_check
class BlueprintsBlockedHealthCheck(HealthCheck):
    name = 'Has no Blocked blueprints'

    @classmethod
    def applicable(cls, card, store):
        return card.status != 'Ready'

    @classmethod
    def check(cls, card, store):
        blueprints = card_blueprints_by_status(store, card.roadmap_id)
        return len(blueprints['Blocked']) == 0


@register_health_check
class RoadmapIdHealthCheck(HealthCheck):
    name = 'Has a roadmap id'

    @classmethod
    def applicable(cls, card, store=None):
        return True

    @classmethod
    def check(cls, card, store=None):
        return card.roadmap_id != ''
